@@ -29,12 +29,12 @@ Grammar:
 """
 
 import ast
+import builtins
 import glob
 import os
 import sys
 import tokenize
-from io import open
-import builtins
+from functools import partial
 
 from .net import websafe
 from .utils import re_compile, safestr, safeunicode, storage
@@ -73,8 +73,7 @@ def splitline(text):
 
 
 class Parser:
-    """Parser Base.
-    """
+    """Parser Base."""
 
     def __init__(self):
         self.statement_nodes = STATEMENT_NODES
@@ -129,11 +128,11 @@ class Parser:
     def read_var(self, text):
         r"""Reads a var statement.
 
-            >>> read_var = Parser().read_var
-            >>> read_var('var x=10\nfoo')
-            (<var: x = 10>, 'foo')
-            >>> read_var('var x: hello $name\nfoo')
-            (<var: x = join_(u'hello ', escape_(name, True))>, 'foo')
+        >>> read_var = Parser().read_var
+        >>> read_var('var x=10\nfoo')
+        (<var: x = 10>, 'foo')
+        >>> read_var('var x: hello $name\nfoo')
+        (<var: x = join_(u'hello ', escape_(name, True))>, 'foo')
         """
         line, text = splitline(text)
         tokens = self.python_tokens(line)
@@ -167,9 +166,9 @@ class Parser:
     def read_suite(self, text):
         r"""Reads section by section till end of text.
 
-            >>> read_suite = Parser().read_suite
-            >>> read_suite('hello $name\nfoo\n')
-            [<line: [t'hello ', $name, t'\n']>, <line: [t'foo\n']>]
+        >>> read_suite = Parser().read_suite
+        >>> read_suite('hello $name\nfoo\n')
+        [<line: [t'hello ', $name, t'\n']>, <line: [t'foo\n']>]
         """
         sections = []
         while text:
@@ -180,13 +179,13 @@ class Parser:
     def readline(self, text):
         r"""Reads one line from the text. Newline is suppressed if the line ends with \.
 
-            >>> readline = Parser().readline
-            >>> readline('hello $name!\nbye!')
-            (<line: [t'hello ', $name, t'!\n']>, 'bye!')
-            >>> readline('hello $name!\\\nbye!')
-            (<line: [t'hello ', $name, t'!']>, 'bye!')
-            >>> readline('$f()\n\n')
-            (<line: [$f(), t'\n']>, '\n')
+        >>> readline = Parser().readline
+        >>> readline('hello $name!\nbye!')
+        (<line: [t'hello ', $name, t'!\n']>, 'bye!')
+        >>> readline('hello $name!\\\nbye!')
+        (<line: [t'hello ', $name, t'!']>, 'bye!')
+        >>> readline('$f()\n\n')
+        (<line: [$f(), t'\n']>, '\n')
         """
         line, text = splitline(text)
 
@@ -204,11 +203,11 @@ class Parser:
     def read_node(self, text):
         r"""Reads a node from the given text and returns the node and remaining text.
 
-            >>> read_node = Parser().read_node
-            >>> read_node('hello $name')
-            (t'hello ', '$name')
-            >>> read_node('$name')
-            ($name, '')
+        >>> read_node = Parser().read_node
+        >>> read_node('hello $name')
+        (t'hello ', '$name')
+        >>> read_node('$name')
+        ($name, '')
         """
         if text.startswith("$$"):
             return TextNode("$"), text[2:]
@@ -229,9 +228,9 @@ class Parser:
     def read_text(self, text):
         r"""Reads a text node from the given text.
 
-            >>> read_text = Parser().read_text
-            >>> read_text('hello $name')
-            (t'hello ', '$name')
+        >>> read_text = Parser().read_text
+        >>> read_text('hello $name')
+        (t'hello ', '$name')
         """
         index = text.find("$")
         if index < 0:
@@ -304,7 +303,6 @@ class Parser:
                     t = next(tokens)
                     if t.value == end:
                         break
-            return
 
         parens = {"(": ")", "[": "]", "{": "}"}
 
@@ -368,9 +366,9 @@ class Parser:
     def read_assignment(self, text):
         r"""Reads assignment statement from text.
 
-            >>> read_assignment = Parser().read_assignment
-            >>> read_assignment('a = b + 1\nfoo')
-            (<assignment: 'a = b + 1'>, 'foo')
+        >>> read_assignment = Parser().read_assignment
+        >>> read_assignment('a = b + 1\nfoo')
+        (<assignment: 'a = b + 1'>, 'foo')
         """
         line, text = splitline(text)
         return AssignmentNode(line.strip()), text
@@ -378,13 +376,13 @@ class Parser:
     def python_lookahead(self, text):
         """Returns the first python token from the given text.
 
-            >>> python_lookahead = Parser().python_lookahead
-            >>> python_lookahead('for i in range(10):')
-            'for'
-            >>> python_lookahead('else:')
-            'else'
-            >>> python_lookahead(' x = 1')
-            ' '
+        >>> python_lookahead = Parser().python_lookahead
+        >>> python_lookahead('for i in range(10):')
+        'for'
+        >>> python_lookahead('else:')
+        'elsweb/template.py
+        >>> python_lookahead(' x = 1')
+        ' '
         """
         i = iter([text])
         readline = lambda: next(i)
@@ -427,9 +425,9 @@ class Parser:
     def read_statement(self, text):
         r"""Reads a python statement.
 
-            >>> read_statement = Parser().read_statement
-            >>> read_statement('for i in range(10): hello $name')
-            ('for i in range(10):', ' hello $name')
+        >>> read_statement = Parser().read_statement
+        >>> read_statement('for i in range(10): hello $name')
+        ('for i in range(10):', ' hello $name')
         """
         tok = PythonTokenizer(text)
         tok.consume_till(":")
@@ -498,12 +496,12 @@ class PythonTokenizer:
     def consume_till(self, delim):
         """Consumes tokens till colon.
 
-            >>> tok = PythonTokenizer('for i in range(10): hello $i')
-            >>> tok.consume_till(':')
-            >>> tok.text[:tok.index]
-            'for i in range(10):'
-            >>> tok.text[tok.index:]
-            ' hello $i'
+        >>> tok = PythonTokenizer('for i in range(10): hello $i')
+        >>> tok.consume_till(':')
+        >>> tok.text[:tok.index]
+        'for i in range(10):'
+        >>> tok.text[tok.index:]
+        ' hello $i'
         """
         try:
             while True:
@@ -558,7 +556,7 @@ class DefwithNode:
         return encoding + self.defwith + self.suite.emit(indent + INDENT) + self.end
 
     def __repr__(self):
-        return "<defwith: %s, %s>" % (self.defwith, self.suite)
+        return f"<defwith: {self.defwith}, {self.suite}>"
 
 
 class TextNode:
@@ -583,14 +581,14 @@ class ExpressionNode:
         self.escape = escape
 
     def emit(self, indent, begin_indent=""):
-        return "escape_(%s, %s)" % (self.value, bool(self.escape))
+        return f"escape_({self.value}, {bool(self.escape)})"
 
     def __repr__(self):
         if self.escape:
             escape = ""
         else:
             escape = ":"
-        return "$%s%s" % (escape, self.value)
+        return f"${escape}{self.value}"
 
 
 class AssignmentNode:
@@ -634,7 +632,7 @@ class BlockNode:
         return out
 
     def __repr__(self):
-        return "<block: %s, %s>" % (repr(self.stmt), repr(self.suite))
+        return f"<block: {repr(self.stmt)}, {repr(self.suite)}>"
 
 
 class ForNode(BlockNode):
@@ -648,7 +646,7 @@ class ForNode(BlockNode):
         BlockNode.__init__(self, stmt, block, begin_indent)
 
     def __repr__(self):
-        return "<block: %s, %s>" % (repr(self.original_stmt), repr(self.suite))
+        return f"<block: {repr(self.original_stmt)}, {repr(self.suite)}>"
 
 
 class CodeNode:
@@ -713,10 +711,10 @@ class VarNode:
         self.value = value
 
     def emit(self, indent, text_indent):
-        return indent + "self[%s] = %s\n" % (repr(self.name), self.value)
+        return indent + f"self[{repr(self.name)}] = {self.value}\n"
 
     def __repr__(self):
-        return "<var: %s = %s>" % (self.name, self.value)
+        return f"<var: {self.name} = {self.value}>"
 
 
 class SuiteNode:
@@ -776,19 +774,18 @@ TEMPLATE_BUILTIN_NAMES = [
     "ord",
     "pow",
     "range",
+    "round",
     "True",
     "False",
     "None",
     "__import__",  # some c-libraries like datetime requires __import__ to present in the namespace
 ]
 
-TEMPLATE_BUILTINS = dict(
-    [
-        (name, getattr(builtins, name))
-        for name in TEMPLATE_BUILTIN_NAMES
-        if name in builtins.__dict__
-    ]
-)
+TEMPLATE_BUILTINS = {
+    name: getattr(builtins, name)
+    for name in TEMPLATE_BUILTIN_NAMES
+    if name in builtins.__dict__
+}
 
 
 class ForLoop:
@@ -829,8 +826,7 @@ class ForLoop:
 
 
 class ForLoopContext:
-    """Stackable context for ForLoop to support nested for loops.
-    """
+    """Stackable context for ForLoop to support nested for loops."""
 
     def __init__(self, forloop, parent):
         self._forloop = forloop
@@ -879,10 +875,21 @@ class BaseTemplate:
         __hidetraceback__ = True  # noqa: F841
         return self.t(*a, **kw)
 
-    def make_env(self, globals, builtins):
+    def make_env(self, globals, builtins_):
+        if sys.implementation.name == "pypy":
+            # Pypy's `__builtins__` can't be overridden in exec. More details see issue #598.
+            overridden_builtins = builtins.__dict__.keys() - builtins_.keys()
+
+            def f(name, *args, **kwargs):
+                raise NameError("name '%s' is not defined" % name)
+
+            for name in overridden_builtins:
+                if name not in globals:
+                    globals[name] = partial(f, name)
+
         return dict(
             globals,
-            __builtins__=builtins,
+            __builtins__=builtins_,
             ForLoop=ForLoop,
             TemplateResult=TemplateResult,
             escape_=self._escape,
@@ -890,7 +897,7 @@ class BaseTemplate:
         )
 
     def _join(self, *items):
-        return u"".join(items)
+        return "".join(items)
 
     def _escape(self, value, escape=False):
         if value is None:
@@ -942,6 +949,13 @@ class Template(BaseTemplate):
             builtins=builtins,
         )
 
+    def __repr__(self):
+        """
+        >>> Template(text='Template text', filename='burndown_chart.html')
+        <Template burndown_chart.html>
+        """
+        return f"<{self.__class__.__name__} {self.filename}>"
+
     def normalize_text(text):
         """Normalizes template text by correcting \r\n, tabs and BOM chars."""
         text = text.replace("\r\n", "\n").replace("\r", "\n").expandtabs()
@@ -1002,11 +1016,7 @@ class Template(BaseTemplate):
             compiled_code = compile(code, filename, "exec")
         except SyntaxError as err:
             # display template line that caused the error along with the traceback.
-            err.msg += "\n\nTemplate traceback:\n    File %s, line %s\n        %s" % (
-                repr(err.filename),
-                err.lineno,
-                get_source_line(err.filename, err.lineno - 1),
-            )
+            err.msg += f"\n\nTemplate traceback:\n    File {repr(err.filename)}, line {err.lineno}\n        {get_source_line(err.filename, err.lineno - 1)}"
 
             raise
 
@@ -1086,9 +1096,8 @@ class Render:
                 path, cache=self._cache is not None, base=self._base, **self._keywords
             )
         elif kind == "file":
-            return Template(
-                open(path, encoding="utf-8").read(), filename=path, **self._keywords
-            )
+            with open(path, encoding="utf-8") as tmpl_file:
+                return Template(tmpl_file.read(), filename=path, **self._keywords)
         else:
             raise AttributeError("No template named " + name)
 
@@ -1167,8 +1176,7 @@ except ImportError:
 
 
 def frender(path, **keywords):
-    """Creates a template from the given file path.
-    """
+    """Creates a template from the given file path."""
     return Template(open(path, encoding="utf-8").read(), filename=path, **keywords)
 
 
@@ -1212,8 +1220,8 @@ def compile_templates(root):
             out.write(code)
 
             out.write("\n\n")
-            out.write("%s = CompiledTemplate(%s, %s)\n" % (name, name, repr(path)))
-            out.write("join_ = %s._join; escape_ = %s._escape\n\n" % (name, name))
+            out.write(f"{name} = CompiledTemplate({name}, {repr(path)})\n")
+            out.write(f"join_ = {name}._join; escape_ = {name}._escape\n\n")
 
             # create template to make sure it compiles
             Template(open(path, encoding="utf-8").read(), path)
@@ -1337,7 +1345,7 @@ class SafeVisitor(ast.NodeVisitor):
 
     def __init__(self, *args, **kwargs):
         "Initialize visitor by generating callbacks for all AST node types."
-        super(SafeVisitor, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.errors = []
 
     def walk(self, tree, filename):
@@ -1351,13 +1359,13 @@ class SafeVisitor(ast.NodeVisitor):
         nodename = type(node).__name__
         if nodename not in ALLOWED_AST_NODES:
             self.fail_name(node, nodename)
-        super(SafeVisitor, self).generic_visit(node)
+        super().generic_visit(node)
 
     def visit_Attribute(self, node):
         attrname = self.get_node_attr(node)
         if self.is_unallowed_attr(attrname):
             self.fail_attribute(node, attrname)
-        super(SafeVisitor, self).generic_visit(node)
+        super().generic_visit(node)
 
     def visit_Assign(self, node):
         self.check_assign_targets(node)
@@ -1368,7 +1376,7 @@ class SafeVisitor(ast.NodeVisitor):
     def check_assign_targets(self, node):
         for target in node.targets:
             self.check_assign_target(target)
-        super(SafeVisitor, self).generic_visit(node)
+        super().generic_visit(node)
 
     def check_assign_target(self, targetnode):
         targetname = type(targetnode).__name__
@@ -1433,7 +1441,7 @@ class TemplateResult(MutableMapping):
 
     def __init__(self, *a, **kw):
         self.__dict__["_d"] = dict(*a, **kw)
-        self._d.setdefault("__body__", u"")
+        self._d.setdefault("__body__", "")
 
         self.__dict__["_parts"] = []
         self.__dict__["extend"] = self._parts.extend
@@ -1444,10 +1452,9 @@ class TemplateResult(MutableMapping):
         return self._d.keys()
 
     def _prepare_body(self):
-        """Prepare value of __body__ by joining parts.
-        """
+        """Prepare value of __body__ by joining parts."""
         if self._parts:
-            value = u"".join(self._parts)
+            value = "".join(self._parts)
             self._parts[:] = []
             body = self._d.get("__body__")
             if body:
