@@ -2,23 +2,17 @@
 Web API (wrapper around WSGI)
 (from web.py)
 """
-from __future__ import print_function
 
-import cgi
 import pprint
 import sys
-from io import BytesIO
+import urllib
+from http.cookies import CookieError, Morsel, SimpleCookie
+from urllib.parse import parse_qs, quote, unquote, urljoin
+
+import multipart
 
-from .py3helpers import PY2, text_type, urljoin
 from .utils import dictadd, intget, safestr, storage, storify, threadeddict
 
-try:
-    from urllib.parse import unquote, quote
-    from http.cookies import CookieError, Morsel, SimpleCookie
-except ImportError:
-    from urllib import unquote, quote
-    from Cookie import CookieError, Morsel, SimpleCookie
-
 __all__ = [
     "config",
     "header",
@@ -237,8 +231,7 @@ class _NotFound(HTTPError):
 
 
 def NotFound(message=None):
-    """Returns HTTPError with '404 Not Found' error from the active application.
-    """
+    """Returns HTTPError with '404 Not Found' error from the active application."""
     if message:
         return _NotFound(message)
     elif ctx.get("app_stack"):
@@ -353,8 +346,7 @@ class _UnavailableForLegalReasons(HTTPError):
 
 
 def UnavailableForLegalReasons(message=None):
-    """Returns HTTPError with '415 Unavailable For Legal Reasons' error from the active application.
-    """
+    """Returns HTTPError with '415 Unavailable For Legal Reasons' error from the active application."""
     if message:
         return _UnavailableForLegalReasons(message)
     elif ctx.get("app_stack"):
@@ -378,8 +370,7 @@ class _InternalError(HTTPError):
 
 
 def InternalError(message=None):
-    """Returns HTTPError with '500 internal error' error from the active application.
-    """
+    """Returns HTTPError with '500 internal error' error from the active application."""
     if message:
         return _InternalError(message)
     elif ctx.get("app_stack"):
@@ -411,52 +402,64 @@ def header(hdr, value, unique=False):
 
 
 def rawinput(method=None):
-    """Returns storage object with GET or POST arguments.
-    """
+    """Returns storage object with GET or POST arguments."""
     method = method or "both"
 
     def dictify(fs):
-        # hack to make web.input work with enctype='text/plain.
-        if fs.list is None:
-            fs.list = []
+        return {k: fs[k] for k in fs}
 
-        return dict([(k, fs[k]) for k in fs])
-
-    e = ctx.env.copy()
+    env = ctx.env.copy()
     a = b = {}
 
+    # This is required by the multipart module and not set up by the tests, so add the default value if missing
+    # srb: not sure if thtis is needed, as OpenLibrary appears to work without it. It also changes the content
+    #      type for many requests. Change tests, or add this? Not sure.
+    # if "CONTENT_TYPE" not in env:
+    #     env["CONTENT_TYPE"] = "application/x-www-form-urlencoded"
+
     if method.lower() in ["both", "post", "put", "patch"]:
-        if e["REQUEST_METHOD"] in ["POST", "PUT", "PATCH"]:
-            if e.get("CONTENT_TYPE", "").lower().startswith("multipart/"):
-                # since wsgi.input is directly passed to cgi.FieldStorage,
-                # it can not be called multiple times. Saving the FieldStorage
+        if env["REQUEST_METHOD"] in ["POST", "PUT", "PATCH"]:
+            if env.get("CONTENT_TYPE", "").lower().startswith("multipart/"):
+                # since wsgi.input is directly passed to multipart,
+                # it can not be called multiple times. Saving the result
                 # object in ctx to allow calling web.input multiple times.
-                a = ctx.get("_fieldstorage")
+                a = ctx.get(
+                    "_fieldstorage"
+                )  # TODO: Rename? is this visible anywhere else?
                 if not a:
-                    fp = e["wsgi.input"]
-                    a = cgi.FieldStorage(fp=fp, environ=e, keep_blank_values=1)
-                    ctx._fieldstorage = a
+                    try:
+                        # This returns two dicts, forms & files.
+                        forms, a = multipart.parse_form_data(environ=env)
+                        a = dictadd(forms, a)
+                        ctx._fieldstorage = a
+                    except IndexError:
+                        a = {}
+
             else:
-                d = data()
-                if PY2 and isinstance(d, text_type):
-                    d = d.encode("utf-8")
-                fp = BytesIO(d)
-                a = cgi.FieldStorage(fp=fp, environ=e, keep_blank_values=1)
+                post_data = data().decode("utf-8")
+                a = parse_qs(post_data, keep_blank_values=True)
+                # TODO: Can we safely ignore files here or should we merge them
+                # This returns two dicts: forms & files
+                # a, files = multipart.parse_form_data(environ=env)
             a = dictify(a)
 
     if method.lower() in ["both", "get"]:
-        e["REQUEST_METHOD"] = "GET"
-        b = dictify(cgi.FieldStorage(environ=e, keep_blank_values=1))
+        env["REQUEST_METHOD"] = "GET"
+        b = dict(
+            urllib.parse.parse_qs(env.get("QUERY_STRING", ""), keep_blank_values=True)
+        )
 
-    def process_fieldstorage(fs):
-        if isinstance(fs, list):
-            return [process_fieldstorage(x) for x in fs]
-        elif fs.filename is None:
-            return fs.value
+    def process_values(values):
+        if isinstance(values, list):
+            return [process_values(x) for x in values]
+        elif (
+            hasattr(values, "filename") and values.filename is None
+        ):  # FIXME this probably needs to be improved
+            return values.value
         else:
-            return fs
+            return values
 
-    return storage([(k, process_fieldstorage(v)) for k, v in dictadd(b, a).items()])
+    return storage([(k, process_values(v)) for k, v in dictadd(b, a).items()])
 
 
 def input(*requireds, **defaults):
@@ -509,36 +512,11 @@ def setcookie(
     if httponly:
         morsel["httponly"] = True
     value = morsel.OutputString()
-    if samesite and samesite.lower() in ("strict", "lax"):
+    if samesite and samesite.lower() in ("strict", "lax", "none"):
         value += "; SameSite=%s" % samesite
     header("Set-Cookie", value)
 
 
-def decode_cookie(value):
-    r"""Safely decodes a cookie value to unicode.
-
-    Tries us-ascii, utf-8 and io8859 encodings, in that order.
-
-    >>> decode_cookie('')
-    u''
-    >>> decode_cookie('asdf')
-    u'asdf'
-    >>> decode_cookie('foo \xC3\xA9 bar')
-    u'foo \xe9 bar'
-    >>> decode_cookie('foo \xE9 bar')
-    u'foo \xe9 bar'
-    """
-    try:
-        # First try plain ASCII encoding
-        return text_type(value, "us-ascii")
-    except UnicodeError:
-        # Then try UTF-8, and if that fails, ISO8859
-        try:
-            return text_type(value, "utf-8")
-        except UnicodeError:
-            return text_type(value, "iso8859", "ignore")
-
-
 def parse_cookies(http_cookie):
     r"""Parse a HTTP_COOKIE header and return dict of cookie names and decoded values.
 
@@ -553,7 +531,7 @@ def parse_cookies(http_cookie):
     >>> sorted(parse_cookies('a=1; b=2; c=3').items())
     [('a', '1'), ('b', '2'), ('c', '3')]
 
-    # TODO: cclauss reenable this test
+    # TODO: cclauss re-enable this test
     # >>> sorted(parse_cookies('a=1; b=w("x")|y=z; c=3').items())
     # [('a', '1'), ('b', 'w('), ('c', '3')]
 
@@ -580,7 +558,7 @@ def parse_cookies(http_cookie):
                     cookie.load(attr_value)
                 except CookieError:
                     pass
-        cookies = dict([(k, unquote(v.value)) for k, v in cookie.items()])
+        cookies = {k: unquote(v.value) for k, v in cookie.items()}
     else:
         # HTTP_COOKIE doesn't have quotes, use fast cookie parsing
         cookies = {}
@@ -602,10 +580,6 @@ def cookies(*requireds, **defaults):
 
     The values are converted to unicode if _unicode=True is passed.
     """
-    # If _unicode=True is specified, use decode_cookie to convert cookie value to unicode
-    if defaults.get("_unicode") is True:
-        defaults["_unicode"] = decode_cookie
-
     # parse cookie string and cache the result for next time.
     if "_parsed_cookies" not in ctx:
         http_cookie = ctx.env.get("HTTP_COOKIE", "")
